import java.io.*;
import java.util.*;

public class CopyTest {
    private StreamFileCopier streamCopier;
    private FileCopier1 copier1;
    private FileCopier2 copier2;
    private FileCopier3 copier3;

    private List<String> files;

    public CopyTest() throws IOException {
        streamCopier = new StreamFileCopier();
        copier1 = new FileCopier1();
        copier2 = new FileCopier2();
        copier3 = new FileCopier3();

        prepare();
    }

    private void prepare() throws IOException {
        files = new ArrayList<String>();

        int[] bases = {1, 2, 5};
        for (long i = 1000; i < 100000000L; i *= 10L) {
            for (int base: bases) {
                long length = i * base;
                String filename = length + ".txt";
                makeFile(filename, length);
                files.add(filename);
            }
        }
            String filename = "100000000.txt";
            makeFile(filename, 100000000);
            files.add(filename);
    }

    private void makeFile(String filename, long size) throws IOException {
        BufferedOutputStream stream = null;

        try {
            stream = new BufferedOutputStream(new FileOutputStream(filename));
            
            for (long l = 0; l < size; l++) {
                stream.write('a');
            }
        } finally {
            stream.close();
        }
    }

    public void test(String selection) throws IOException {
        if (selection.equals("s")) {
            streamTest();
        } else if (selection.equals("1")) {
            nioTest1();
        } else if (selection.equals("2")) {
            nioTest2();
        } else if (selection.equals("3")) {
            nioTest3();
        } else {
            streamTest();
            nioTest1();
            nioTest2();
            nioTest3();
        }
    }

    private void streamTest() throws IOException {
        System.out.println("Stream");
        for (String file: files) {
            for (int i = 0; i < 10 ; i++) {
                streamCopier.copy(file, file + ".bak");
            }

            long start = System.currentTimeMillis();
            for (int i = 0; i < 10 ; i++) {
                streamCopier.copy(file, file + ".bak");
            }
            long end = System.currentTimeMillis();
            System.out.println(file + " " + (end - start));
        }
    }

    private void nioTest1() throws IOException {
        System.out.println("NIO 1");
        for (String file: files) {
            for (int i = 0; i < 10 ; i++) {
                copier1.copy(file, file + ".bak");
            }

            long start = System.currentTimeMillis();
            for (int i = 0; i < 10 ; i++) {
                copier1.copy(file, file + ".bak");
            }
            long end = System.currentTimeMillis();
            System.out.println(file + " " + (end - start));
        }
    }

    private void nioTest2() throws IOException {
        System.out.println("NIO 2");
        for (String file: files) {
            for (int i = 0; i < 10 ; i++) {
                copier2.copy(file, file + ".bak");
            }

            long start = System.currentTimeMillis();
            for (int i = 0; i < 10 ; i++) {
                copier2.copy(file, file + ".bak");
            }
            long end = System.currentTimeMillis();
            System.out.println(file + " " + (end - start));
        }
    }

    private void nioTest3() throws IOException {
        System.out.println("NIO 3");
        for (String file: files) {
            for (int i = 0; i < 10 ; i++) {
                copier3.copy(file, file + ".bak");
            }

            long start = System.currentTimeMillis();
            for (int i = 0; i < 10 ; i++) {
                copier3.copy(file, file + ".bak");
            }
            long end = System.currentTimeMillis();
            System.out.println(file + " " + (end - start));
        }
    }

    public static void main(String[] args) {
        try {
            CopyTest test = new CopyTest();
            test.test(args[0]);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
}
